/*
 * Project Spitfire/SDL
----------------------------------------------------------------------
	sprite.h - Sprite engine for use with cs.h
----------------------------------------------------------------------
 * © David Olofson, 2001
 */

#ifndef _SPRITE_H_
#define _SPRITE_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "SDL.h"

typedef enum
{
	S_BLITMODE_NORMAL = 0,	/* Use source alpha if present */
	S_BLITMODE_OPAQUE,	/* Alpha: blend over black during conv. */
	S_BLITMODE_COLORKEY,	/* Like OPAQUE, but apply colorkey to the result */
	S_BLITMODE_ALPHA	/* Apply source alpha during conv,
				 * then use full surface alpha */
} s_blitmodes_t;

/* sprite image */
typedef struct
{
	SDL_Surface	*surface;
} s_sprite_t;

/* Bank of sprite images */
typedef struct
{
	unsigned	w, h;	/* Same size for all sprites in bank */
	unsigned	last;
	unsigned	max;
	s_sprite_t	**sprites;
} s_bank_t;

/* Two dimensionally indexed sprite container */
typedef struct
{
	unsigned	max;
	s_bank_t	**banks;
} s_container_t;


/*
 * Basic Container Management
 */
s_container_t *s_new_container(unsigned banks);
void s_delete_container(s_container_t *c);

s_bank_t *s_new_bank(s_container_t *c, unsigned bank, unsigned frames,
				unsigned w, unsigned h);
void s_delete_bank(s_container_t *c, unsigned bank);
void s_delete_all_banks(s_container_t *c);

s_sprite_t *s_new_sprite(s_container_t *c, unsigned bank, unsigned frame);
void s_delete_sprite(s_container_t *c, unsigned bank, unsigned frame);

/*
 * Getting data
 */
s_sprite_t *s_get_sprite(s_container_t *c, unsigned bank, unsigned frame);
s_sprite_t *s_get_sprite_b(s_bank_t *b, unsigned frame);

/*
 * File Import Tools
 */
int s_load_image(s_container_t *c, unsigned bank, const char *name);
int s_load_bank(s_container_t *c, unsigned bank, unsigned w, unsigned h,
						const char *name);
int s_load_sprite(s_container_t *c, unsigned bank, unsigned frame,
						const char *name);

/*
 * Lower level interfaces
 */
s_bank_t *s_get_bank(s_container_t *c, unsigned bank);
s_sprite_t *s_new_sprite_b(s_bank_t *b, unsigned frame);
void s_delete_sprite_b(s_bank_t *b, unsigned frame);

/*
 * UURGH!!!
 */
extern s_blitmodes_t	s_blitmode;
extern unsigned long	s_colorkey;
extern unsigned char	s_alpha;

#ifdef __cplusplus
};
#endif

#endif /* _SPRITE_H_ */
