
/*
 * XKOBO, a video-oriented game
 * Copyright (C) 1995,1996  Akira Higuchi
 *     a-higuti@math.hokudai.ac.jp
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#include "skobo.h"
#include "screen.h"
#include "manage.h"
#include "enemies.h"
#include "myship.h"
#include "key.h"
#include "radar.h"
#include "scenes.h"
#include "config.h"
#include "random.h"
#include "version.h"

int _screen::scene_max;
int _screen::scene_num;
int _screen::level;
int _screen::generate_count;
_map _screen::map;
int _screen::pixel_f0 = -1;
int _screen::pixel_f1 = -1;
int _screen::pixel_f2 = -1;
int _screen::pixel_s[8];
int _screen::show_title = 0;

/* Translate "map code" into tile index */
int _screen::translate(int n)
{
	if(IS_SPACE(n))
		return (n & 0xff) + 128;

	int x = n;
	int y = 7;
	if(n & HARD)
	{
		y = 6;
		if(n & U_MASK)
			x = 8;
		else if(n & R_MASK)
			x = 9;
		else if(n & D_MASK)
			x = 10;
		else if(n & L_MASK)
			x = 11;
	}
	else if(n & CORE)
	{
		y = 6;
		x = 15;
	}
	return y * 16 + x;
}


void _screen::init()
{
	scene_max = 0;
	while(scene[scene_max].ratio != -1)
		scene_max++;
	if(pixel_f0 < 0)
	{
		pixel_f0 = wbase.alloc_color(200 << 8, 255 << 8, 255 << 8);
		pixel_f1 = wbase.alloc_color(140 << 8, 200 << 8, 200 << 8);
		pixel_f2 = wbase.alloc_color(100 << 8, 200 << 8, 100 << 8);
		int i;
		for(i = 0; i < 8; i++)
		{
			pixel_s[i] = wbase.alloc_color(40000,
					15000 + i * 4000,
					50000 - i * 5000);
		}
	}
}


/*
FIXME: This should be replaced with some nice scrolling stuff.
*/
void _screen::title()
{
//	char s[256];
//	manage.get_highscore(s);
//	wchip.foreground(pixel_f0);
//	wchip.font(B_BIG_FONT);
//	wchip.center(40, "SKOBO");
	wchip.foreground(pixel_f1);
	wchip.sprite((wchip.width() - 208)/2, 40, B_LOGO, 0);
	wchip.font(B_NORMAL_FONT);
	wchip.center(115, KOBO_VERSION);
//	wchip.center(100, KOBO_DATE);
	if(cheat_mode)
		wchip.center(130, "CHEAT MODE");
//	else
//		wchip.center(140, s);
	wchip.foreground(pixel_f2);
	wchip.center(160, "Press 'SPACE' to Play");
	wchip.center(175, "'+'/'-' to Select Stage");
	wchip.center(190, "'ESC' to Quit");
}


void _screen::init_scene(int sc)
{
	map.init();
	if(sc < 0)
	{
		/*
		 * Intro mode
		 */
//		scene_num = -1;
		show_title = 1;
		myship.off();
		enemies.off();
		if(sc == INTRO_SCENE)
		{
			/*
			 * Plain intro - no map
			 */
			radar.prepare();	/* Clear radar */
			scene_num = 19 % scene_max;
			level = 19 / scene_max;
		}
		else
		{
			/*
			 * Map selection - show current map
			 */
			scene_num = -(sc+1) % scene_max;
			level = -(sc+1) / scene_max;
		}
	}
	else
	{
		/*
		 * In-game mode
		 */
		gengine.period(0);
		show_title = 0;
		scene_num = sc % scene_max;
		level = sc / scene_max;
	}
//printf("scene: %d, level: %d\n", scene_num, level);

	_scene *s = &scene[scene_num];
	int i;
	for(i = 0; i < s->base_max; i++)
		map.make_maze(s->base[i].x, s->base[i].y, s->base[i].h,
				s->base[i].v);
	map.convert(s->ratio);

	/* No radar map in intro mode! */
	if(sc != INTRO_SCENE)
		radar.prepare();

	generate_count = 0;
}


int _screen::prepare()
{
	if(scene_num < 0)
		return 0;
	_scene *s = &scene[scene_num];
	int i, j;
	int count_core = 0;
	int c = 0;

	int interval_1, interval_2;
	interval_1 = (s->ek1_interval) >> level;
	interval_2 = (s->ek2_interval) >> level;
	if(interval_1 < 4)
		interval_1 = 4;
	if(interval_2 < 4)
		interval_2 = 4;
	enemies.set_ekind_to_generate(s->ek1, interval_1, s->ek2,
			interval_2);

	wchip.clear();
	for(i = 0; i < MAP_SIZEX; i++)
		for(j = 0; j < MAP_SIZEY; j++)
		{
			int m = map.pos(i, j);
			if((m == U_MASK) || (m == R_MASK) || (m == D_MASK)
					|| (m == L_MASK))
			{
				enemies.make(&cannon, i * 16 + 8,
						j * 16 + 8);
				c++;
			}
			else if(m == CORE)
			{
				enemies.make(&core, i * 16 + 8,
						j * 16 + 8);
				count_core++;
				c++;
			}
		}

#if 0
	int x, y;
	for(i = 0; i < 8; i++)
	{
		wchip.foreground(pixel_s[i]);
		for(j = 0; j < 2000; j++)
		{
			x = rand_num.get(WORLD_SIZEX_LOG2);
			y = rand_num.get(WORLD_SIZEY_LOG2);
			if(IS_SPACE(map.pos(x >> 4, y >> 4)))
				wchip.fillrectangle(x, y, 1, 1);
//                wchip.fillrectangle_back(x, y, 1, 1);
		}
	}
#endif

	myship.set_position(s->startx << 4, s->starty << 4);

	radar.prepare();
	return count_core;
}


void _screen::generate_fixed_enemies()
{
	static int sint[16] =
			{ 0, 12, 23, 30, 32, 30, 23, 12, 0, -12, -23, -30,
				-32, -30, -23, -12 };
	static int cost[16] =
			{ 32, 30, 23, 12, 0, -12, -23, -30, -32, -30, -23,
				-12, 0, 12, 23, 30 };
	_scene *s = &scene[scene_num];
	if(generate_count < s->enemy_max)
	{
		int j;
		for(j = 0; j < s->enemy[generate_count].num; j++)
		{
			int sp = s->enemy[generate_count].speed;
			int x, y, h, v, t;
			x = rand_num.get() % (WORLD_SIZEX - VIEWLIMIT * 2);
			y = rand_num.get() % (WORLD_SIZEY - VIEWLIMIT * 2);
			x -= (WORLD_SIZEX / 2 - VIEWLIMIT);
			y -= (WORLD_SIZEY / 2 - VIEWLIMIT);
			if(x < 0)
				x -= VIEWLIMIT;
			else
				x += VIEWLIMIT;
			if(y < 0)
				y -= VIEWLIMIT;
			else
				y += VIEWLIMIT;
			x += myship.get_x();
			y += myship.get_y();

			t = rand_num.get(4);
			h = PIXEL2CS(sp * sint[t]) / 64;
			v = PIXEL2CS(sp * cost[t]) / 64;
			enemies.make(s->enemy[generate_count].kind, x, y,
					h, v);
		}
		generate_count++;
	}
	if(generate_count >= s->enemy_max)
		generate_count = 0;
}


int _screen::get_chip_number(int x, int y)
{
	return map.pos(x, y);
}


void _screen::set_chip_number(int x, int y, int n)
{
	if(n == 0)
	{
		map.clearpos(x, y);
		radar.erase(x, y);
	}
	else
		map.pos(x, y) = n;

}


void _screen::pre_render(window_t * win)
{
	if(!win)
		return;

	SDL_Rect vr;
	int xo, yo, x, y;
	int mx, my;
	vr.x = CS2PIXEL(gengine.xoffs(LAYER_BASES));
	vr.y = CS2PIXEL(gengine.yoffs(LAYER_BASES));
	vr.w = win->width();
	vr.h = win->height();

	/* 
	 * Start exactly at the top-left corner
	 * of the tile visible in the top-left
	 * corner of the display window.
	 */
	xo = vr.x & (CHIP_SIZEX - 1);
	yo = vr.y & (CHIP_SIZEY - 1);
	mx = vr.x >> CHIP_SIZEX_LOG2;
	my = vr.y >> CHIP_SIZEY_LOG2;

	for(y = 0; y < (WSIZE >> CHIP_SIZEY_LOG2) + 2; ++y)
		for(x = 0; x < (WSIZE >> CHIP_SIZEX_LOG2) + 2; ++x)
		{
			int n = map.pos(mx + x, my + y);
			win->sprite((x << CHIP_SIZEX_LOG2) - xo,
					(y << CHIP_SIZEX_LOG2) - yo,
					0, translate(n));
		}
}


void _screen::post_render(window_t * win)
{
	if(!win)
		return;

	if(show_title)
		title();
	else if(manage.state_paused())
		wchip.center(130, "PAUSED");
}
