#ifndef ___AUDIO_H_
#define ___AUDIO_H_

#ifdef __cplusplus
extern "C" {
#endif

#define MAX_CHANNEL	16
#define DEFAULT_VOL	100

#define	SAMPLE_RATE	8000

#define SAMP_MIN -128
#define SAMP_MAX 127

/*
 * Changes:
 * play_stop(-1) feature removed.
 *	Silent channels are now released automatically.
 *		/David
 *
 * get_audio_volume() removed.
 *	I cannot safely be used the way KXobo used it
 *	due to the asynchronous nature of the modified
 *	audio engine. (Background music attenuation is
 *	now done in a more reliable way.)
 *		/David
 *
 * play_loop() removed.
 *	Not needed with a callback or thread driven
 *	audio engine.
 *		/David
 *
 * API cleaned up; all function names now begin with
 * 'audio_'.
 *
 * audio_control() API added.
 *		/David
 */

typedef enum
{
	AC_FIRST = 0,
	AC_ERROR = -1,

	AC_PLAYRESET,		//boolean
	AC_LOOP,		//boolean
	AC_RETRIG,		//samples
	AC_RANDTRIG,		//samples

	AC_COUNT,
	AC_LAST = AC_COUNT - 1,
} audio_ctls_t;

/*
 * Init/close
 */
int audio_open(int latency, int use_oss);
void audio_close(void);
void audio_master_volume(int vol);	/* 0 .. 200 */

/*
 * Channel control
 */
void audio_volume(int cid, int vol);	/* def: 100 */
void audio_stop(int cid);	/* -1 mean all */
void audio_play(int cid, char* samp, int nsamp);
void audio_control(int cid, int ctl, int arg);
int audio_is_active(int cid);

#ifdef __cplusplus
};
#endif

#endif /* ___AUDIO_H_ */
